@@ -1,42 +1,66 @@
-from Products.PloneTestCase import PloneTestCase
-from plone.dexterity.fti import DexterityFTI
+# -*- coding: utf-8 -*-
+from five.intid import site
+from plone.app.testing import setRoles
+from plone.app.testing import TEST_USER_ID
+from plone.app.testing import TEST_USER_ROLES
 from plone.app.versioningbehavior.modifiers import CloneNamedFileBlobs
 from plone.app.versioningbehavior.modifiers import SkipRelations
-from unittest import TestSuite, makeSuite
-from plone.namedfile.file import NamedBlobFile
+from plone.app.versioningbehavior.testing import PLONE_APP_VERSIONINGBEHAVIOR_INTEGRATION_TESTING
+from plone.autoform.interfaces import IFormFieldProvider
+from plone.dexterity.fti import DexterityFTI
+from plone.dexterity.utils import createContent
+from plone.dexterity.utils import createContentInContainer
 from plone.namedfile import field
-from plone.directives import form
-from plone.dexterity.utils import createContentInContainer, createContent
-from ZODB.interfaces import IBlob
-from zope.interface import alsoProvides, Interface
-from StringIO import StringIO
-from zope.configuration import xmlconfig
-from zope.component import getUtility
-from z3c.relationfield.schema import RelationChoice, RelationList
+from plone.namedfile.file import NamedBlobFile
+from plone.supermodel import model
+from six import StringIO
 from z3c.relationfield.relation import RelationValue
+from z3c.relationfield.schema import RelationChoice
+from z3c.relationfield.schema import RelationList
+from ZODB.interfaces import IBlob
+from zope.component import getUtility
+from zope.configuration import xmlconfig
+from zope.interface import alsoProvides
+from zope.interface import Interface
+from zope.intid.interfaces import IIntIds
 
-PloneTestCase.setupPloneSite(
-    extension_profiles=['plone.app.versioningbehavior:default'])
+import unittest
 
 
-class IBlobFile(form.Schema):
-        file = field.NamedBlobFile(title=u'File')
-alsoProvides(IBlobFile, form.IFormFieldProvider)
+class IBlobFile(model.Schema):
+    file = field.NamedBlobFile(title=u'File')
+
+
+alsoProvides(IBlobFile, IFormFieldProvider)
 
 
 class IRelationsType(Interface):
-    single = RelationChoice(title=u"Single",
+    single = RelationChoice(title=u'Single',
                             required=False, values=[])
-    multiple = RelationList(title=u"Multiple (Relations field)",
+    multiple = RelationList(title=u'Multiple (Relations field)',
                             required=False)
 
 
-class TestModifiers(PloneTestCase.PloneTestCase):
+class IRelationsBehavior(model.Schema):
+    single = RelationChoice(title=u'Single',
+                            required=False, values=[])
+    multiple = RelationList(title=u'Multiple (Relations field)',
+                            required=False)
 
-    def afterSetUp(self):
+
+alsoProvides(IRelationsBehavior, IFormFieldProvider)
+
+
+class TestModifiers(unittest.TestCase):
+
+    layer = PLONE_APP_VERSIONINGBEHAVIOR_INTEGRATION_TESTING
+
+    def setUp(self):
+        self.portal = self.layer['portal']
+        self.request = self.layer['request']
         # we need to have the Manager role to be able to add things
         # to the portal root
-        self.setRoles(['Manager', ])
+        setRoles(self.portal, TEST_USER_ID, ['Manager'])
 
     def testCloneNamedFileBlobsInSchema(self):
         file_fti = DexterityFTI(
@@ -44,7 +68,8 @@ class TestModifiers(PloneTestCase.PloneTestCase):
             model_source="""
             <model xmlns="http://namespaces.plone.org/supermodel/schema">
                 <schema>
-                    <field name="file" type="plone.namedfile.field.NamedBlobFile">
+                    <field name="file"
+                           type="plone.namedfile.field.NamedBlobFile">
                         <title>File</title>
                         <required>True</required>
                     </field>
@@ -60,7 +85,7 @@ class TestModifiers(PloneTestCase.PloneTestCase):
         self.assertTrue(
             'plone.dexterity.schema.generated.plone_0_BlobFile.file'
             in attrs_dict)
-        blob = attrs_dict.values()[0]
+        blob = list(attrs_dict.values())[0]
         self.assertTrue(IBlob.providedBy(blob))
 
         file2 = createContentInContainer(self.portal, 'BlobFile')
@@ -101,7 +126,7 @@ class TestModifiers(PloneTestCase.PloneTestCase):
         self.assertTrue(
             'plone.app.versioningbehavior.tests.test_modifiers.IBlobFile.file'
             in attrs_dict)
-        blob = attrs_dict.values()[0]
+        blob = list(attrs_dict.values())[0]
         self.assertTrue(IBlob.providedBy(blob))
 
         file2 = createContentInContainer(self.portal, 'BlobFile')
@@ -116,7 +141,8 @@ class TestModifiers(PloneTestCase.PloneTestCase):
             model_source="""
             <model xmlns="http://namespaces.plone.org/supermodel/schema">
                 <schema>
-                    <field name="file" type="plone.namedfile.field.NamedBlobFile">
+                    <field name="file"
+                           type="plone.namedfile.field.NamedBlobFile">
                         <title>File</title>
                         <required>True</required>
                     </field>
@@ -128,7 +154,8 @@ class TestModifiers(PloneTestCase.PloneTestCase):
         file1 = createContentInContainer(self.portal, 'BlobFile')
         file1.file = NamedBlobFile('dummy test data', filename=u'test.txt')
         modifier = CloneNamedFileBlobs('modifier', 'Modifier')
-        pers_id, pers_load, empty1, empty2 = modifier.getOnCloneModifiers(file1)
+        on_clone_modifiers = modifier.getOnCloneModifiers(file1)
+        pers_id, pers_load, empty1, empty2 = on_clone_modifiers
         self.assertTrue(pers_id(file1.file._blob))
         self.assertTrue(pers_load(file1.file._blob) is None)
         self.assertTrue(empty1 == [])
@@ -140,7 +167,8 @@ class TestModifiers(PloneTestCase.PloneTestCase):
             model_source="""
             <model xmlns="http://namespaces.plone.org/supermodel/schema">
                 <schema>
-                    <field name="file" type="plone.namedfile.field.NamedBlobFile">
+                    <field name="file"
+                           type="plone.namedfile.field.NamedBlobFile">
                         <title>File</title>
                         <required>True</required>
                     </field>
@@ -152,7 +180,8 @@ class TestModifiers(PloneTestCase.PloneTestCase):
         modifier = CloneNamedFileBlobs('modifier', 'Modifier')
         attrs_dict = modifier.getReferencedAttributes(file1)
         self.assertTrue(attrs_dict == {})
-        pers_id, pers_load, empty1, empty2 = modifier.getOnCloneModifiers(file1)
+        on_clone_modifiers = modifier.getOnCloneModifiers(file1)
+        pers_id, pers_load, empty1, empty2 = on_clone_modifiers
         self.assertTrue(pers_id(None) is None)
         self.assertTrue(pers_load(None) is None)
         self.assertTrue(empty1 == [])
@@ -165,9 +194,10 @@ class TestModifiers(PloneTestCase.PloneTestCase):
         self.assertTrue(
             'plone.dexterity.schema.generated.plone_0_BlobFile.file'
             in attrs_dict)
-        blob = attrs_dict.values()[0]
+        blob = list(attrs_dict.values())[0]
         self.assertTrue(IBlob.providedBy(blob))
-        pers_id, pers_load, empty1, empty2 = modifier.getOnCloneModifiers(file1)
+        on_clone_modifiers = modifier.getOnCloneModifiers(file1)
+        pers_id, pers_load, empty1, empty2 = on_clone_modifiers
         self.assertTrue(pers_id(file1.file._blob))
         self.assertTrue(pers_load(file1.file._blob) is None)
         self.assertTrue(empty1 == [])
@@ -176,17 +206,32 @@ class TestModifiers(PloneTestCase.PloneTestCase):
     def testRelations(self):
         rel_fti = DexterityFTI(
             'RelationsType',
-            schema="plone.app.versioningbehavior.tests.test_modifiers.IRelationsType")
+            schema=IRelationsType.__identifier__
+        )
         self.portal.portal_types._setObject('RelationsType', rel_fti)
 
-        # Setup IIntIds utility which is required for relations to work
-        from five.intid import site
-        from zope.app.intid.interfaces import IIntIds
-        site.add_intids(self.portal)
         intids = getUtility(IIntIds)
 
         source = createContentInContainer(self.portal, 'RelationsType')
         target = createContentInContainer(self.portal, 'RelationsType')
+
+        # Test modifier when no relations are set
+        modifier = SkipRelations('modifier', 'Modifier')
+        on_clone_modifiers = modifier.getOnCloneModifiers(source)
+        pers_id, pers_load, empty1, empty2 = on_clone_modifiers
+        self.assertTrue(pers_id(None) is None)
+        self.assertTrue(pers_id(None) is None)
+        self.assertTrue(pers_load(None) is None)
+        self.assertTrue(pers_load(None) is None)
+        self.assertTrue(empty1 == [])
+        self.assertTrue(empty2 == [])
+
+        repo_clone = createContent('RelationsType')
+        modifier.afterRetrieveModifier(source, repo_clone)
+        self.assertTrue(repo_clone.single is source.single)
+        self.assertTrue(repo_clone.multiple is source.multiple)
+
+        # Add some relations
         source.single = RelationValue(intids.getId(target))
         source.multiple = [RelationValue(intids.getId(target))]
 
@@ -196,7 +241,8 @@ class TestModifiers(PloneTestCase.PloneTestCase):
         notify(ObjectModifiedEvent(source))
 
         modifier = SkipRelations('modifier', 'Modifier')
-        pers_id, pers_load, empty1, empty2 = modifier.getOnCloneModifiers(source)
+        on_clone_modifiers = modifier.getOnCloneModifiers(source)
+        pers_id, pers_load, empty1, empty2 = on_clone_modifiers
         self.assertTrue(pers_id(source.single))
         self.assertTrue(pers_id(source.multiple))
         self.assertTrue(pers_load(source.single) is None)
@@ -209,8 +255,100 @@ class TestModifiers(PloneTestCase.PloneTestCase):
         self.assertTrue(repo_clone.single is source.single)
         self.assertTrue(repo_clone.multiple is source.multiple)
 
+    def register_RelationsType(self):
+        xmlconfig.xmlconfig(StringIO(
+            '''
+            <configure
+                 package="plone.behavior"
+                 xmlns="http://namespaces.zope.org/zope"
+                 xmlns:plone="http://namespaces.plone.org/plone"
+                 i18n_domain="plone.behavior.tests">
 
-def test_suite():
-    suite = TestSuite()
-    suite.addTest(makeSuite(TestModifiers))
-    return suite
+                 <include package="plone.behavior" file="meta.zcml" />
+
+                <plone:behavior
+                    title="Relations behavior"
+                    description="A behavior"
+                    provides="plone.app.versioningbehavior.tests.test_modifiers.IRelationsBehavior"
+                    />
+            </configure>
+            '''
+        ))
+        rel_fti = DexterityFTI(
+            'RelationsType',
+            behaviors=[IRelationsBehavior.__identifier__]
+        )
+        self.portal.portal_types._setObject('RelationsType', rel_fti)
+
+    def testRelationsInBehaviors(self):
+        self.register_RelationsType()
+        intids = getUtility(IIntIds)
+
+        source = createContentInContainer(self.portal, 'RelationsType')
+        target = createContentInContainer(self.portal, 'RelationsType')
+
+        # Test modifier when no relations are set
+        modifier = SkipRelations('modifier', 'Modifier')
+        on_clone_modifiers = modifier.getOnCloneModifiers(source)
+        pers_id, pers_load, empty1, empty2 = on_clone_modifiers
+        self.assertTrue(pers_id(None) is None)
+        self.assertTrue(pers_id(None) is None)
+        self.assertTrue(pers_load(None) is None)
+        self.assertTrue(pers_load(None) is None)
+        self.assertTrue(empty1 == [])
+        self.assertTrue(empty2 == [])
+
+        repo_clone = createContent('RelationsType')
+        modifier.afterRetrieveModifier(source, repo_clone)
+        self.assertTrue(repo_clone.single is None)
+        self.assertTrue(repo_clone.multiple is None)
+
+        # Add some relations
+        IRelationsBehavior(source).single = RelationValue(
+            intids.getId(target)
+        )
+        IRelationsBehavior(source).multiple = [
+            RelationValue(intids.getId(target))
+        ]
+
+        # Update relations
+        from zope.lifecycleevent import ObjectModifiedEvent
+        from zope.event import notify
+        notify(ObjectModifiedEvent(source))
+
+        modifier = SkipRelations('modifier', 'Modifier')
+        on_clone_modifiers = modifier.getOnCloneModifiers(source)
+        pers_id, pers_load, empty1, empty2 = on_clone_modifiers
+        self.assertTrue(pers_id(IRelationsBehavior(source).single))
+        self.assertTrue(pers_id(IRelationsBehavior(source).multiple))
+        self.assertTrue(pers_load(IRelationsBehavior(source).single) is None)
+        self.assertTrue(pers_load(IRelationsBehavior(source).multiple) is None)
+        self.assertTrue(empty1 == [])
+        self.assertTrue(empty2 == [])
+
+        repo_clone = createContent('RelationsType')
+        modifier.afterRetrieveModifier(source, repo_clone)
+        self.assertTrue(IRelationsBehavior(repo_clone).single
+                        is IRelationsBehavior(source).single)
+        self.assertTrue(IRelationsBehavior(repo_clone).multiple
+                        is IRelationsBehavior(source).multiple)
+
+    def testRelationsInBehaviorsForMigratedDXObjects(self):
+        ''' Do not break in the case of
+        dexterity objects with relations migrated from something else
+        (e.g. Archetypes)
+        '''
+        self.register_RelationsType()
+        source = createContentInContainer(self.portal, 'RelationsType')
+
+        # Test modifier when no relations are set
+        class Dummy(object):
+            pass
+
+        repo_clone = Dummy()
+
+        modifier = SkipRelations('modifier', 'Modifier')
+        modifier.afterRetrieveModifier(source, repo_clone)
+
+        self.assertFalse(hasattr(repo_clone, 'single'))
+        self.assertFalse(hasattr(repo_clone, 'multiple'))
